<?php

class CourseMeta {
	private $users = array();
	private $course_table, $students_table, $invitations_table, $subscriptions_table,$student_groups_table;
	private $wpdb;

	function __construct($cat_id, $accept_key = false, $wpdb_prefix_override = false) {
		global $wpdb;
		$this->cat_ID = $cat_id;
		if($wpdb_prefix_override) {
			if(is_multisite()) {
				global $blog_id;
				if($blog_id > 1) {
					$prefix = $wpdb->base_prefix.$blog_id.'_LePressTeacherRole_';
				}
			} 
		}
		
		if(!isSet($prefix)) {
			$prefix = teacher_wpdb_prefix;
		}
		$this->course_table = $prefix.'course_meta';
		$this->students_table = $prefix.'students';
		$this->student_groups_table = $prefix.'student_groups';
		$this->invitations_table = $prefix.'invitations';
		$this->subscriptions_table = $prefix.'subscriptions';
		$this->teachers_table = $prefix.'teachers';
		$this->wpdb = $wpdb;
		$this->accept_key = $accept_key;
		if(!$this->metaDataExist()) {
			$this->wpdb->insert($this->course_table, array('cat_id' => $cat_id));
		}
		/* Runs everytime to clean out expired invitations keys */
		$this->removeInviteKeys();
	}
	
	function unsubcribeAllStudents() {
		$active_subscriptions = $this->getApprovedSubscriptions();
		$pending_subscriptions = $this->getPendingSubscriptions();
		
		foreach($active_subscriptions as $subs_act) {
			 $this->setSubscriptionStatus($subs_act->id, 2, $message = false);
		}
		
		foreach($pending_subscriptions as $subs_pend) {
			$this->setSubscriptionStatus($subs_pend->id, 2, $message = false);
		}
		
	}
	
	function sendNotification($to, $subject, $message) {
		$current_user = wp_get_current_user();
		$headers = 'From: '.$current_user->user_firstname.' '.$current_user->user_lastname.' <'.$current_user->user_email.'>' . "\r\n";	
		wp_mail($to, $subject, $message, $header);
	}
	
	function verifyAccess($blog_url, $email, $action) {
		if($this->getIsCourse()) {
			//First check if we are not trying to subscribe on our own course
			if($blog_url == get_bloginfo('siteurl')) {
				return -3;
			}
			if(($subs = $this->getStudentSubscriptionByBlogUrl($blog_url, true)) || !$this->studentExist($email)) {
				if($action == 'subscribe' && $subs) {
					return -2;
				}
				return $blog_url;
			} else {
				if($action == 'subscribe') {
					return $blog_url;
				}
			}
		} else {
			return -1;
		}
		return false;
	}

	function getName() {
		$cat = get_category($this->cat_ID);
		return $cat->cat_name;
	}

	private function metaDataExist() {
		return $this->wpdb->get_row('SELECT * FROM '.$this->course_table.' WHERE cat_id = '.esc_sql($this->cat_ID));
	}

	function setIsCourse($bool) {
		$this->wpdb->update($this->course_table, array('is_course' => $bool), array('cat_id' => $this->cat_ID));
	}

	function getIsCourse() {
		return (bool) $this->wpdb->get_var('SELECT is_course FROM '.$this->course_table.' WHERE cat_id = '.esc_sql($this->cat_ID));
	}
	
	/** Wordpress Network used only */
	
	function getAdvertise() {
		return (bool) $this->wpdb->get_var('SELECT advertise FROM '.$this->course_table.' WHERE cat_id = '.esc_sql($this->cat_ID));
	}
	
	function setAdvertise($bool) {
		$this->wpdb->update($this->course_table, array('advertise' => $bool), array('cat_id' => $this->cat_ID));
	}
	
	/** End of Wordpress Network used only */

	function setAccessType($bool) {
		$this->wpdb->update($this->course_table, array('access_type' => $bool), array('cat_id' => $this->cat_ID));
	}

	function getAccessType() {
		return (bool) $this->wpdb->get_var('SELECT access_type FROM '.$this->course_table.' WHERE cat_id = '.esc_sql($this->cat_ID));
	}

	function setTeachers($teachers) {
		//Add additional teachers
		$current_user = wp_get_current_user();
		if(is_array($teachers)) {
			foreach($teachers as $user_id => $action) {
				$teacher = get_userdata($user_id);
				if($teacher) {
					if($action == 0) {
						$result = $this->wpdb->query('DELETE FROM '.$this->teachers_table.' WHERE cat_id = '.esc_sql($this->cat_ID).' AND wp_user_id='.esc_sql($user_id));
						if($result) {
							$title = __('You are no longer a LePress teacher', lepress_textdomain);
							$message = __('You are no longer a LePress teacher on blog', lepress_textdomain).' - '.get_bloginfo('siteurl')."\r\n";
						}
					} else {
						$data = array('cat_id' => $this->cat_ID, 'wp_user_id' => $user_id);
						$result = $this->wpdb->insert($this->teachers_table, $data);
						if($user_id != $current_user->ID) {
							if($result) {
								$title = __('You have been added as a LePress teacher', lepress_textdomain);
								$message = __('You have been added as a LePress teacher on blog', lepress_textdomain).' - '.get_bloginfo('siteurl')."\r\n";
							}
						}
					}
					if(!empty($message) && $user_id != $current_user->ID) {
						//Send email notification to new teacher
						$this->sendNotification($teacher->user_email, $title, $message);
					}
				}
			}
		}

		//If editing category and category has no teachers, add current user as a teacher
		if(!$this->getTeachers(true)) {
			global $current_user;
			get_currentuserinfo();
			$data = array('cat_id' => $this->cat_ID, 'wp_user_id' => $current_user->ID);
			$this->wpdb->insert($this->teachers_table, $data);
		}
	}

	function getTeachers($onlyCat_ID = false) {
		if($onlyCat_ID) {
			return $this->wpdb->get_results('SELECT wp_user_id  FROM '.$this->teachers_table.' WHERE '.$this->teachers_table.'.cat_id = '.esc_sql($this->cat_ID));	
		} else {
			return $this->wpdb->get_results('SELECT wp_user_id  FROM '.$this->teachers_table.', '.$this->subscriptions_table.' WHERE  accept_key = "'.esc_sql($this->accept_key).'" AND '.$this->teachers_table.'.cat_id = '.$this->subscriptions_table.'.cat_id  AND '.$this->subscriptions_table.'.cat_id = '.esc_sql($this->cat_ID));
		}
	}
	
	function getTeacherCats($user_id) {
		return $this->wpdb->get_results('SELECT cat_id  FROM '.$this->teachers_table.' WHERE '.$this->teachers_table.'.wp_user_id = '.esc_sql($user_id));
	}

	function isTeacher($wp_user_id) {
		return $this->wpdb->get_var('SELECT wp_user_id  FROM '.$this->teachers_table.' WHERE '.$this->teachers_table.'.cat_id = '.esc_sql($this->cat_ID).' AND wp_user_id = '.esc_sql($wp_user_id));
	}

	function delete() {
		return $this->wpdb->query('DELETE FROM '.$this->course_table.' WHERE cat_id = '.esc_sql($this->cat_ID));
	}

	function validateInviteKey($key, $email) {
		return $this->wpdb->get_var('SELECT id FROM '.$this->invitations_table.' WHERE cat_id = '.esc_sql($this->cat_ID).' AND new_student_email = "'.esc_sql($email).'" AND invite_key = "'.esc_sql($key).'" AND UNIX_TIMESTAMP(DATE_ADD(expire_time, INTERVAL 7 day)) >= UNIX_TIMESTAMP(NOW())');
	}

	function removeInviteKeys() {
		/* Removes only expired invitation keys */
		$this->wpdb->query('DELETE FROM '.$this->invitations_table.' WHERE UNIX_TIMESTAMP(DATE_ADD(expire_time, INTERVAL 7 day)) <= UNIX_TIMESTAMP(NOW())');
	}

	function getStudentBySubscription($subscription_id) {
		return $this->wpdb->get_row('SELECT '.$this->students_table.'.id, first_name, last_name, blog_url, email, accept_key FROM '.$this->subscriptions_table.', '.$this->students_table.' WHERE '.$this->subscriptions_table.'.id = '.$subscription_id.' AND student_id = '.$this->students_table.'.id AND cat_id = '.esc_sql($this->cat_ID));
	}

	function getStudentSubscriptionByBlogUrl($blog_url, $ignore_accept_key = false) {
		if($ignore_accept_key) {
			return $this->wpdb->get_row('SELECT '.$this->subscriptions_table.'.id, first_name, last_name, blog_url, email, accept_key FROM '.$this->subscriptions_table.', '.$this->students_table.' WHERE blog_url = "'.esc_sql($blog_url).'" AND cat_id = '.esc_sql($this->cat_ID).' AND student_id = '.$this->students_table.'.id');
		} else {
			return $this->wpdb->get_row('SELECT '.$this->subscriptions_table.'.id, first_name, last_name, blog_url, email, accept_key FROM '.$this->subscriptions_table.', '.$this->students_table.' WHERE blog_url = "'.esc_sql($blog_url).'" AND accept_key = "'.esc_sql($this->accept_key).'" AND cat_id = '.esc_sql($this->cat_ID).' AND student_id = '.$this->students_table.'.id');
		}
	}

	function getStudentHasSubscription($student) {
		return $this->wpdb->get_var('SELECT count(id) FROM '.$this->subscriptions_table.' WHERE student_id = '.esc_sql($student->id));
	}

	function removeSubscription($blog_url, $accept_key, $subscription_id = false) {
		if(!$subscription_id) {
			$subscription_id = $this->wpdb->get_var('SELECT '.$this->subscriptions_table.'.id FROM '.$this->subscriptions_table.', '.$this->students_table.' WHERE blog_url = "'.esc_sql($blog_url).'" AND student_id = '.$this->students_table.'.id AND accept_key = "'.esc_sql($accept_key).'"');
		}
		if($subscription_id) {
			$student = $this->getStudentBySubscription($subscription_id);
			$deleted = $this->wpdb->query('DELETE FROM '.$this->subscriptions_table.' WHERE ID = "'.esc_sql($subscription_id).'" AND accept_key = "'.esc_sql($accept_key).'"');
			if($deleted) {
				$this->sendNotification($student->email, __("You have been unsubscribed from course", lepress_textdomain), __("You have been unsubscribed from course", lepress_textdomain)." ".get_category_link($this->cat_ID));
				//Check, if students has no subscriptions left
				$student_has_subscriptions = $this->getStudentHasSubscription($student);
				if(!$student_has_subscriptions) {
					$this->removeStudent($student);
				}
				return true;
			} else {
				return false;
			}
		} else {
			return true;
		}
	}

	function addSubscription($firstname, $lastname, $email, $blog_url, $message, $invite_key = false) {
		$status = (int) $this->getAccessType();
		if($invite_key) {
			if(!$this->validateInviteKey($invite_key, $email) && $status == 0) {
				return false;
			} else {
				$status = 1;
			}
		}
		if(!$this->studentExist($email)) {
			$this->addStudent($firstname, $lastname, $email, $blog_url);
		}
		if($student_id = $this->studentExist($email)) {
			$accept_key = $this->generateAcceptKey();
			$result = $this->wpdb->insert($this->subscriptions_table, array('cat_id' => $this->cat_ID, 'student_id' => $student_id, 'status' => $status, 'accept_key' => $accept_key, 'message' => $message));
			if($result) {				
				$this->accept_key = $accept_key; //Need to set accept_key in order to getTeachers	
				$this->sendNotification($email, "You have successfully subscribed to course", "You have successfully subscribed to course ".get_category_link($this->cat_ID));
				//return course metadata for request
				return $this->getCourseMeta(array('course-status' => $status, 'accept-key' => $accept_key, 'lepress-message' => $message));
			} else {
				//UNIQUE OR FOREIGN KEY CONSTRAINT, student already subscribed
				return -2;
			}
		} else {
			//Add student has failed ?
			return -3;
		}
		return -4;
	}
	

	private function addStudent($firstname, $lastname, $email, $blog_url) {
		return $this->wpdb->insert($this->students_table, array('first_name' => $firstname,
																				'last_name' =>	$lastname,
																				'email' => $email,
																				'blog_url' => $blog_url));
	}

	private function removeStudent($student) {
		$this->wpdb->query('DELETE  FROM '.$this->students_table.' WHERE id = "'.esc_sql($student->id).'"');
	}
	private function studentExist($email) {
		return $this->wpdb->get_var('SELECT id FROM '.$this->students_table.' WHERE email = "'.esc_sql($email).'"');
	}

	private function studentAlreadySubscribed($email) {
		return $this->wpdb->get_results('SELECT '.$this->subscriptions_table.'.id, first_name, last_name, email FROM '.$this->students_table.', '.$this->subscriptions_table.' WHERE student_id = '.$this->students_table.'.id AND email = "'.esc_sql($email).'" AND cat_id = '.esc_sql($this->cat_ID));
	}
	
	public function getStudentSubscriptionByEmail($email) {
		$s = $this->studentAlreadySubscribed($email);
		return $s[0];
	}

	private function generateAcceptKey($length = 10) {
		$chars = "abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789";
		$key = "";
		for ($i=0; $i<$length; $i++) {
			$key .= substr( $chars, rand( 0, strlen( $chars ) - 1), 1 );
		}
		return $key;
	}

	function getApprovedCount() {
		return $this->wpdb->get_var('SELECT count(id) FROM '.$this->subscriptions_table.' WHERE cat_id = '.esc_sql($this->cat_ID).' AND status = 1');
	}

	function getPendingCount() {
		return $this->wpdb->get_var('SELECT count(id) FROM '.$this->subscriptions_table.' WHERE cat_id = '.esc_sql($this->cat_ID).' AND status = 0');
	}

	function getAllPendingSubscriptionsCount() {
		$cats = get_categories('hide_empty=0');
		$count = 0;
		foreach($cats as $cat) {
			$this->cat_ID = $cat->cat_ID;
			if($this->getIsCourse()) {
				if($subscriptions = $this->getPendingSubscriptions()) {
					foreach($subscriptions as $user) {
						$count++;
					}
				}
			}
		}
		return $count;
	}

	function getAllUngradedAssignmentCount() {
		$count = 0;
		$cats = get_categories('hide_empty=0');
		foreach($cats as $cat) {
			$this->cat_ID = $cat->cat_ID;
			if($this->getIsCourse()) {
				$assignments = get_posts(array('meta_query' => array(array('key' => '_is-lepress-assignment', 'value' => 1)), 'post_status' => 'draft|future|publish', 'numberposts' => -1, 'category' => $this->cat_ID));
				foreach($assignments as $post) {
					$count = $count + $this->getUngradedCount($post->ID);
				}
			}
		}
		return $count;
	}

	function getPendingSubscriptions() {
		return $this->wpdb->get_results('SELECT '.$this->subscriptions_table.'.id, first_name, last_name, email, message, blog_url FROM '.$this->students_table.', '.$this->subscriptions_table.' WHERE student_id = '.$this->students_table.'.id AND status = 0 AND cat_id = '.esc_sql($this->cat_ID).' ORDER BY last_name ASC');
	}

	function getApprovedSubscriptions() {
		return $this->wpdb->get_results('SELECT '.$this->subscriptions_table.'.id, '.$this->students_table.'.id as student_id, first_name, last_name, email, blog_url FROM '.$this->students_table.', '.$this->subscriptions_table.' WHERE student_id = '.$this->students_table.'.id AND status = 1 AND cat_id = '.esc_sql($this->cat_ID).' ORDER BY last_name ASC');
	}
	
	function hasSubscriptions() {
		if($this->getApprovedSubscriptions() || $this->getPendingSubscriptions()) {
			return true;
		}
		return false;
	}

	function getUngradedCount($post_id) {
		global $LePress;
		$ungraded = 0;
		foreach($this->getApprovedSubscriptions() as $subscription){
			$comment = $LePress->fetchComment($post_id, $subscription->email, 'LePressStudent');
			$meta_data = get_post_meta($post_id, '_lepress-student-'.md5($post_id.$subscription->email), true);
			if($comment && !$meta_data['ungraded']) {
				$ungraded++;
			}
		}
		return $ungraded;
	}

	function setSubscriptionStatus($subscription_id, $status, $message = false) {
		$student = $this->getStudentBySubscription($subscription_id);

		$data = array('status' => $status);
		$where = array('id' => $subscription_id);
		switch($status) {
			case 1:
				$updated = $this->wpdb->update($this->subscriptions_table, $data, $where);
				$this->sendNotification($student->email, __("You have been accepted to course", lepress_textdomain), __("You have been accepted to course", lepress_textdomain)." ".get_category_link($this->cat_ID)."\r\n\r\n".$message);
				break;
			case 0:
			case 2:
				$updated = $this->wpdb->query('DELETE FROM '.$this->subscriptions_table.' WHERE id = "'.esc_sql($subscription_id).'"');
				if($status == 2) {
					$this->sendNotification($student->email, __("You have been unsubscribed from course", lepress_textdomain), __("You have been unsubscribed from course", lepress_textdomain)." ".get_category_link($this->cat_ID));
				}
				break;
		}
		if($updated) {
			if($student) {
				$request = new LePressRequest(1, 'subscription-status-changed');
				$request->addParams(array('course-url' =>get_category_link($this->cat_ID), 'course-status' => $status, 'accept-key' => $student->accept_key, 'lepress-message' => $message));
				$request->setBlocking('false');
				$request->doPost($student->blog_url);
				if($status == 0) {
					$this->sendNotification($student->email, __("You have been declined to course", lepress_textdomain), __("You have been declined to course", lepress_textdomain)." ".get_category_link($this->cat_ID)."\r\n\r\n".$message);
				}
			}
			//Check, if students has no subscriptions left
			$student_has_subscriptions = $this->getStudentHasSubscription($student);
			if(!$student_has_subscriptions) {
				$this->removeStudent($student);
			}
		}
	}

	function addAnswer($post_id, $answer_url, $answer_content, $answer_post_id, $blog_url) {
		global $LePress;
		$post = get_post($post_id);
		if($post) {
			$post_cats = wp_get_post_categories($post->ID);
			if(get_post_meta($post->ID, '_is-lepress-assignment', true) && in_array($this->cat_ID, $post_cats)) {
				$subscription = $this->getStudentSubscriptionByBlogUrl($blog_url);
				if($subscription) {
					$comment = $LePress->fetchComment($post->ID, $subscription->email, 'LePressStudent');
					$time = current_time('mysql');
					$data = array(
					    'comment_post_ID' => $post_id,
					    'comment_author' => $subscription->last_name.' '.$subscription->first_name,
					    'comment_author_email' => $subscription->email,
					    'comment_author_url' => $answer_url,
					    'comment_content' => $LePress->mb_cutstr(htmlspecialchars_decode($answer_content))."\n\n".'<a href="'.$answer_url.'">'.urldecode($answer_url).'</a>',
					    'comment_agent' => 'LePressStudent',
					    'comment_type' => '',
					    'comment_date' => $time,
					    'comment_parent' => 0,
					    'user_id' => 0,
					    'comment_approved' => 1);
					if(!$comment) {
						$comment_id = wp_insert_comment($data);
						update_comment_meta($comment_id, 'lepress-read', '1');
						update_post_meta($post->ID, '_lepress-student-'.md5($post->ID.$subscription->email), array('ungraded' => false, 'post_url' => $answer_url, 'post_id' => $answer_post_id));
					} else {
						//First check metadata status
						$lepress_read = get_comment_meta($comment->comment_ID, 'lepress-read', true);
						//Fetch feedback given meta
						$feedback_given = get_comment_meta($comment->comment_ID, 'lepress-feedback-given', true);
						//Delete previous comment and add new with old metadata
						$deleted = wp_delete_comment($comment->comment_ID, true);
						if($deleted) {
							$comment_id = wp_insert_comment($data);
							if($lepress_read) {
								update_comment_meta($comment_id, 'lepress-read', 1);
							}
							if($feedback_given) {
								update_comment_meta($comment_id, 'lepress-feedback-given', 1);
							}
						}
					}
				}
			}
		}
	}

	function sendFeedback($post_id, $student, $student_answer, $feedback, $grade) {
		$grade = strtoupper($grade);
		if($student_answer) {
			$attrs = $student_answer->attributes();
			$student_post_id = (int) $attrs['id'];
			global $current_user, $LePress;
			if($this->isTeacher($current_user->ID)) {			
				$request = new LePressRequest(1, 'addFeedback');
				$request->addParams(array('course-url' => get_category_link($this->cat_ID), 'accept-key' => $student->accept_key, 'teacher_id' => $current_user->ID, 'post_id' => $student_post_id, 'feedback' => $LePress->safeStringInput($feedback), 'grade' => $LePress->safeStringInput($grade)));
				$request->doPost($student->blog_url);
	
				$md5_key = md5($post_id.$student->email);
				$meta_data = get_post_meta($post_id, '_lepress-student-'.$md5_key, true);
				if($meta_data) {
					$meta_data['ungraded'] = true;
					$meta_data['grade'] = $grade;
					$meta_data['feedback'] = $feedback;
					update_post_meta($post_id,  '_lepress-student-'.$md5_key, $meta_data);
				}
				return $request->getStatusCode();
			} else {
				return -1;
			}
		}
	}

	function getStudentAssignmentAnswer($student, $post_id) {
		$meta_data = get_post_meta($post_id, '_lepress-student-'.md5($post_id.$student->email), true);
		if($meta_data['post_id']){
			$request = new LePressRequest(1, 'getAnswerByID');
			$request->addParams(array('course-url' => get_category_link($this->cat_ID), 'accept-key' => $student->accept_key, 'post_id' => $meta_data['post_id']));
			$request->doPost($student->blog_url);
			if($request->getStatusCode() == 200) {
				$xml = @simplexml_load_string($request->getBody())->answer;
				if($xml) {
					return $xml;
				}
				return false;
			}
		}
		return false;
	}

	function getCourseMeta($extra_nodes = array()) {
		global $LePress;
		$xml = new SimpleXMLElement(xml_root);
		$course = $xml->addChild('courseMeta');
		$cat = get_category($this->cat_ID);
		$course->addChild('name', '<![CDATA['.$LePress->safeStringInput($cat->name).']]>');
		$course->addChild('url', '<![CDATA['.$LePress->safeStringInput(get_category_link($this->cat_ID)).']]>');
		if(count($extra_nodes) > 0) {
			$extras = $course->addChild('extras');
			foreach($extra_nodes as $node => $value) {
				$extras->addChild($node, '<![CDATA['.$LePress->safeStringInput($value).']]>');
			}
		}

		$teachers = $course->addChild('teachers');
		foreach($this->getTeachers() as $row) {
			$wp_user_id = $row->wp_user_id;
			$userdata = get_userdata($wp_user_id);
			$teacher = $teachers->addChild('teacher');
			$teacher->addAttribute('id', $wp_user_id);
			$teacher->addChild('firstname', '<![CDATA['.$LePress->safeStringInput((!empty($userdata->user_firstname) ? $userdata->user_firstname : $userdata->user_login)).']]>');
			$teacher->addChild('lastname', '<![CDATA['.$LePress->safeStringInput($userdata->user_lastname).']]>');
			$teacher->addChild('email', '<![CDATA['.$LePress->safeStringInput($userdata->user_email).']]>');
			$teacher->addChild('organization', '<![CDATA['.$LePress->safeStringInput(get_the_author_meta('lepress-organization', $wp_user_id)).']]>');
		}
		return htmlspecialchars_decode($xml->asXML());
	}

	function getAssignments() {
		global $LePress;
		$posts = get_posts(array('meta_query' => array(array('key' => '_is-lepress-assignment', 'value' => 1)), 'post_status' => 'publish', 'numberposts' => -1, 'category' => $this->cat_ID));
		$xml = new SimpleXMLElement(xml_root);
		foreach($posts as $post) {
			$assignment = $xml->addChild('assignment');
			$assignment->addAttribute('post_id', $post->ID);
			$assignment->addChild('title', '<![CDATA['.$LePress->safeStringInput($post->post_title).']]>');
			$assignment->addChild('url', '<![CDATA['.$LePress->safeStringInput(get_permalink($post->ID)).']]>');
			if(!empty($post->post_excerpt)) {
				$assignment->addChild('excerpt', '<![CDATA['.$LePress->safeStringInput($post->post_excerpt).']]>');
			}
			$assignment->addChild('start_date', get_post_meta($post->ID, '_lepress-assignment-start-date', true));
			$assignment->addChild('end_date', get_post_meta($post->ID, '_lepress-assignment-end-date', true));
		}
		return htmlspecialchars_decode($xml->asXML());
	}

	function getClassmates($blog_url) {
		global $LePress;
		$subscriptions = $this->getApprovedSubscriptions();
		$xml = new SimpleXMLElement(xml_root);
		foreach($subscriptions as $classmate) {
			if(trim($classmate->blog_url) != trim($blog_url)) { //Just in case trim
				$mate = $xml->addChild('classmate');
				$mate->addAttribute('id', $classmate->student_id);
				$mate->addChild('firstname', '<![CDATA['.$LePress->safeStringInput($classmate->first_name).']]>');
				$mate->addChild('lastname', '<![CDATA['.$LePress->safeStringInput($classmate->last_name).']]>');
				$mate->addChild('email', '<![CDATA['.$LePress->safeStringInput($classmate->email).']]>');
				$mate->addChild('blog_url', '<![CDATA['.$LePress->safeStringInput($classmate->blog_url).']]>');
			}
		}
		return htmlspecialchars_decode($xml->asXML());
	}

	function getAssignmentByID($post_id) {
		global $LePress;
		$post = get_post($post_id);
		$xml = new SimpleXMLElement(xml_root);
		if($post && $post_id == $post->ID) {
			if(get_post_meta($post->ID, '_is-lepress-assignment', true)) {
				$assignment = $xml->addChild('assignment');
				$assignment->addAttribute('post_id', $post->ID);
				$assignment->addChild('title', '<![CDATA['.$LePress->safeStringInput($post->post_title).']]>');
				$assignment->addChild('url', '<![CDATA['.$LePress->safeStringInput(get_permalink($post->ID)).']]>');
				if(!empty($post->post_excerpt)) {
					$assignment->addChild('excerpt', '<![CDATA['.$LePress->safeStringInput($post->post_excerpt).']]>');
				}
				$assignment->addChild('content', '<![CDATA['.$LePress->safeStringInput($post->post_content).']]>');
				$assignment->addChild('start_date', get_post_meta($post->ID, '_lepress-assignment-start-date', true));
				$assignment->addChild('end_date', get_post_meta($post->ID, '_lepress-assignment-end-date', true));
			}
		}
		return htmlspecialchars_decode($xml->asXML());
	}

	function inviteStudentMessage($email) {
		$already_subscribed = $this->studentAlreadySubscribed($email);
		if(!$already_subscribed) {
			$invite_key = $this->generateAcceptKey();
			$data = array('invite_key' => $invite_key, 'cat_id' => $this->cat_ID, 'new_student_email' => $email);
			$result = $this->wpdb->insert($this->invitations_table, $data);
			if($result) {
				$message = __('You have been invited to course', lepress_textdomain).' - '.$this->getName()."\n";
				$message .= add_query_arg(array('key' => $invite_key), get_category_link($this->cat_ID))."\n";
				return $message;
			}
		}
		return '';
	}

	function updateStudentProfile($firstname, $lastname, $new_email, $old_email) {
		if($student_id = $this->studentExist($old_email)) {
			$data = array('first_name' => $firstname, 'last_name' => $lastname, 'email' => $new_email);
			$where = array('id' => $student_id);
			$this->wpdb->update($this->students_table, $data, $where);
		}
	}

	function addStudentGroup($group_name, $group_key, $student_email) {
		$student_id = $this->studentExist($student_email);
		if($student_id) {
			$data = array('group_name' => $group_name, 'group_key' => $group_key, 'student_id' => $student_id);
			return $this->wpdb->insert($this->student_groups_table, $data);
		}
		return false;
	}

	function removeStudentGroup($group_key, $student_email) {
		$student_id = $this->studentExist($student_email);
		if($student_id) {
			return $this->wpdb->query('DELETE FROM '.$this->student_groups_table.' WHERE student_id = '.esc_sql($student_id).' AND group_key = "'.esc_sql($group_key).'"');
		}
		return false;
	}

	function getGroupByKey($group_key) {
		return $this->wpdb->get_row('SELECT * FROM '.$this->student_groups_table.' WHERE group_key ="'.esc_sql($group_key).'" AND type = 0 AND !ISNULL(group_name)');
	}

	function addMeToGroup($group_key, $blog_url) {
		$subscription = $this->getStudentSubscriptionByBlogUrl($blog_url);
		if($subscription) {
			$student = $this->getStudentBySubscription($subscription->id);
			$group = $this->getGroupByKey($group_key);
			if($student) {
				$data = array('student_id' => $student->id, 'group_key' => $group_key, 'group_name' => $group->group_name, 'type' => 1);
				print_r($data);
				$this->wpdb->insert($this->student_groups_table, $data);
			}
		}
	}
}