<?php

class LePressService {

	function __construct() {
		add_filter('query_vars', array(&$this, 'service_add_trigger'));
		add_action('template_redirect', array(&$this,'service_trigger_check'));
		add_action('wp_head', array(&$this, 'displayMessage'));
	}
	
	function service_add_trigger($vars) {
		$vars[] = 'lepress-service';
		$vars[] = 'lepress-add';
		return $vars;
	}
	
	function service_trigger_check() {
		if(intval(get_query_var('lepress-service')) == 1) {
			$this->handleRequest(); //This function call ends with exit();
		}
		if($course_url = get_query_var('lepress-add')) {
			$course_url = base64_decode($course_url);
			global $LePressStudent;
			if($LePressStudent instanceof LePressStudentRole) {
				if(!is_user_logged_in()) {
					$redirect = wp_login_url(add_query_arg(array('lepress-add' => base64_encode($course_url)), site_url()));
					header('Location:'.$redirect);
				} else {
					if(current_user_can('edit_posts')) {
						$current_user = wp_get_current_user();
						if(!empty($current_user->user_firstname) && !empty($current_user->user_lastname)) {
							$result = $LePressStudent->subscriptions->subscribe($course_url, '', true);
							if($result) {
								update_option('lepress-simple-subscribe-success', $result);
							} else {
								update_option('lepress-simple-subscribe-success', 0);
							}
						} else {
							update_option('lepress-simple-subscribe-success', 3);
						}
					}
					header('Location:'.site_url());
				}
			} else {
				header('Location:'.site_url());
			}
			exit();
		}
	}
	
	function displayMessage() {
		$flag = get_option('lepress-simple-subscribe-success', -1);
		if($flag > -1) {
			if($flag == 1) {
				$txt = __('You have successfully confirmed your LePress subscription!', lepress_textdomain);
			} elseif($flag == 2) {
				$txt = __('You are already subscribed on this course', lepress_textdomain);
			} elseif($flag == 3) {
				$txt = '<span style="color: #FF0000"><strong>'.__('Subscribing failed', lepress_textdomain).'</strong></span><br />'.__('Your profile is not filled (firstname/lastname)', lepress_textdomain);
			} else {
				$txt = __('Confirming your LePress subscription failed! Try again...', lepress_textdomain);
			}
			echo '<div class="lepress-updated"><h3>'.$txt.'</h3></div>';
			delete_option('lepress-simple-subscribe-success');
		}
	}

	function getRole() {
		return $this->role ? $this->role : false;
	}

	function getResponseRole() {
		return $this->getRole() == 1 ? 2 : 1;
	}

	function getAction() {
		return $this->action ? $this->action : false;
	}

	function handleRequest() {
		/* IF passed above if condition, start the fun */
		if(isSet($_POST)) {
			$this->role = $_POST['lepress-role'] ? strip_tags($_POST['lepress-role']) : false;
			$this->action = $_POST['lepress-action'] ? strip_tags($_POST['lepress-action']) : false;
		}

		
		if(isSet($_POST['simple-subscriber-blog']) || isSet($_POST['lepress_user_blog_id'])) {
			if(isSet($_POST['lepress_user_blog_id'])) {
				if(is_user_logged_in() && is_multisite()) {
					$current_user = wp_get_current_user();
					if($current_user->ID == intval($_POST['lepress_user_ID'])) {
						$blog_id_student = intval($_POST['lepress_user_blog_id']);
						foreach(get_blogs_of_user($current_user->ID) as $blog) {
							if($blog->userblog_id == $blog_id_student) {
								//It really is user's blog, let's switch
								$switched = switch_to_blog($blog_id_student, true);
								if($switched) {
									require_once('classes/student.php');
									$LePressStudent = new LePressStudentRole();
									if($LePressStudent instanceof LePressStudentRole) {
										if(!empty($current_user->user_firstname) && !empty($current_user->user_lastname)) {
											$result = $LePressStudent->subscriptions->subscribe($_POST['course-url'], '', true);
											if($result) {
												echo 1;
											} else {
												echo 0;
											}
										} else {
											echo 2;
										}
									}
								}
								break;
							}
						}
						//Return to current blog
						restore_current_blog();
					}
				}
			} else {
				$blog_url = trim(strip_tags($_POST['simple-subscriber-blog']));
				if(filter_var($blog_url, FILTER_VALIDATE_URL)) {
					$course_ID = intval(trim($_POST['lepress-course-id']));
					if($course_ID > 0) {
						$course_meta = new CourseMeta($course_ID);
						if($course_meta->getIsCourse()) {
							$special_url = add_query_arg(array('lepress-add' => base64_encode(get_category_link($course_ID))), $blog_url);
							header('Location: '.$special_url);
						}
					}
				} else {
					echo 3;
				}
			}
			exit;
		}
		
		//otherwise parse request
		if($this->getRole() == 1) {
			$this->parseTeacherRequest();
		}
		if($this->getRole() == 2) {
			$this->parseStudentRequest();
		}

		if($this->getRole() == 22) { //Student blog to other student blog
			$this->parseMateRequest();
		}
	}

	function parseMateRequest() {
		global $LePressStudent;
		switch($this->getAction()) {
			case 'inviteMeToGroup':
					$LePressStudent->groups->addMeToGroup($_POST['group_name'], $_POST['group_key'], $_POST['course_url']);
				break;
		}
		exit();
	}


	function parseTeacherRequest() {
		global $LePressStudent;
		if($LePressStudent->subscriptions->verifyAccess($_POST['course-url'], $_POST['accept-key'])) {
			switch($this->getAction()) {
				case 'subscription-status-changed':
					$LePressStudent->subscriptions->updateStatus($_POST['course-url'], $_POST['course-status'], $_POST['accept-key'], $_POST['lepress-message']);
					break;
				case 'getAnswerByID':
					if($answer = $LePressStudent->subscriptions->getAnswerByPostID($_POST['post_id'])) {
						echo $answer;
					} else {
						header("HTTP/1.1 404 Not found");
					}
					break;
				case 'addFeedback':
					$LePressStudent->subscriptions->addFeedback($_POST['post_id'], $_POST['teacher_id'], $_POST['accept-key'], $_POST['feedback'], $_POST['grade']);
					break;
			}
		} else {
			header('HTTP/1.1 401 Unauthorized');
		}
		exit();
	}

	function parseStudentRequest() {
		if(($cat_id = get_query_var('cat')) > 0 && class_exists('CourseMeta')) {
			global $LePressTeacher;
			$course_meta = new CourseMeta($cat_id, $_POST['accept-key']);
			$blog_url_access = $course_meta->verifyAccess($_POST['lepress-blog'], $_POST['email'], $this->getAction());
			if($blog_url_access > -1) {
				$blog_url = $blog_url_access;
				switch($this->getAction()) {
					case 'subscribe':
						$firstname = $_POST['firstname'];
						$lastname = $_POST['lastname'];
						$email = $_POST['email'];
						$invite_key = isSet($_POST['invite-key']) ? $_POST['invite-key'] : false;
						$message = $_POST['lepress-message'];
						$result = $course_meta->addSubscription($firstname, $lastname, $email, $blog_url, $message, $invite_key);
						if(!$result && is_bool($result)) {
							//Invite key check failed
							header('HTTP/1.1 401 Unauthorized');
						} elseif($result <= -2) {
							header('HTTP/1.1 500 Internal Server Error');
						} else {
							header('HTTP/1.1 202 Accepted');
							echo $result; // Returning course meta to student
						}
						break;
					case 'getCourseMeta';
						echo $course_meta->getCourseMeta();
						break;
					case 'profileUpdated';
						$course_meta->updateStudentProfile($_POST['first_name'], $_POST['last_name'], $_POST['new_email'], $_POST['old_email']);
						header('HTTP/1.1 202 Accepted');
						break;
					case 'unsubscribe':
						$result = $course_meta->removeSubscription($blog_url, $_POST['accept-key']);
						if($result) {
							header('HTTP/1.1 202 Accepted');
						} else {
							header('HTTP/1.1 500 Internal Server Error');
						}
						break;
					case 'getAssignments':
		   				echo $course_meta->getAssignments();
						break;
					case 'getAssignmentByID':
		   				echo $course_meta->getAssignmentByID($_POST['post-id']);
						break;
					case 'addAnswer':
		   				$course_meta->addAnswer($_POST['post_id'], $_POST['answer-url'], $_POST['answer-content'], $_POST['answer-post-id'], $blog_url);
						break;
					case 'getClassmates':
			   			echo $course_meta->getClassmates($blog_url);
						break;
					/* This is disabled, because development of this feature was halted
					 * probably will be completed in near future
					case 'addMyGroup':
						$result = $course_meta->addStudentGroup($_POST['group_name'], $_POST['group_key'], $_POST['email']);
						if($result) {
							header('HTTP/1.1 201 Created');
						} else {
							header('HTTP/1.1 500 Internal Server Error');
						}
						break;
					case 'addMeToGroup':
						$course_meta->addMeToGroup($_POST['group_key'], $blog_url);
						break;
					case 'removeMyGroup':
						$result = $course_meta->removeStudentGroup($_POST['group_key'], $_POST['email']);
						if(!$result) {
							header('HTTP/1.1 500 Internal Server Error');
						}
						break;
						*/
					}
			} elseif($blog_url_access == -1)  {
				header("HTTP/1.1 404 Not found");
			} elseif($blog_url_access == -2) {
				header("HTTP/1.1 409 Conflict");
			} elseif($blog_url_access == -3) {
				header("HTTP/1.1 400 Bad Request");
			} elseif(!$blog_url_access && is_bool($blog_url_access)) {
				header('HTTP/1.1 412 Precondition Failed');
			}
		} else {
			header("HTTP/1.1 404 Not found");
		}
		exit();
	}
}
?>